"""Core module of logkiss.

Copyright (c) 2025 Taka Suzuki
SPDX-License-Identifier: MIT
See LICENSE for details.

This module provides the core functionality of logkiss,
including logging setup and configuration management.
"""

#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import logging
from pathlib import Path
from typing import Optional, Union, TextIO, Dict, Any
from dataclasses import dataclass
from yaml import safe_load, YAMLError
from logging import FileHandler, LogRecord, StreamHandler, Formatter, Filter
from logging.handlers import TimedRotatingFileHandler, RotatingFileHandler

# --- colorama for Windows compatibility ---
try:
    import colorama

    colorama.init()
except ImportError:
    pass  # coloramaがインストールされていない場合でも動作するように

# Exported functions and classes
__all__ = [
    "KissConsoleHandler",
    "ColoredFormatter",
    "KissLogger",
    "use_console_handler",
    "PathShortenerFilter",
    "setup_from_yaml",
    "setup_from_env",
    "setup",
]

# Debug mode settings
DEBUG = os.environ.get("LOGKISS_DEBUG", "").lower() in ("1", "true", "yes")


def should_skip_config() -> bool:
    """
    Check if configuration loading should be skipped based on LOGKISS_SKIP_CONFIG env var.
    Returns True if skip is requested.
    """
    skip = os.environ.get("LOGKISS_SKIP_CONFIG", "").lower()
    return skip in ("1", "true", "yes")


def find_config_file(explicit_path: Optional[Union[str, Path]] = None) -> Optional[Path]:
    """
    Find the config file path according to priority:
    1. explicit_path (argument)
    2. LOGKISS_CONFIG env var
    3. Default locations (OS dependent)
    Returns Path if found, else None.
    """
    if explicit_path:
        p = Path(explicit_path).expanduser()
        if p.exists():
            return p
        # If explicit but not found, still return the path for error reporting
        return p
    # 2. Env var
    env_path = os.environ.get("LOGKISS_CONFIG")
    if env_path:
        p = Path(env_path).expanduser()
        if p.exists():
            return p
        return p
    # 3. Default locations
    candidates = []
    if sys.platform.startswith("win"):
        appdata = os.environ.get("APPDATA")
        userprofile = os.environ.get("USERPROFILE")
        if appdata:
            candidates.append(Path(appdata) / "logkiss" / "config.yaml")
        if userprofile:
            candidates.append(Path(userprofile) / ".config" / "logkiss" / "config.yaml")
    else:
        home = Path.home()
        candidates.append(home / ".config" / "logkiss" / "config.yaml")
        candidates.append(Path.cwd() / "logkiss.yaml")
    for candidate in candidates:
        if candidate.exists():
            return candidate
    return None


def setup(explicit_config: Optional[Union[str, Path]] = None) -> logging.Logger:
    """
    Unified setup function for logkiss.
    - Honors LOGKISS_SKIP_CONFIG to skip config loading
    - Searches config file in documented order
    - Applies config if found, else uses environment/defaults
    Args:
        explicit_config: Path to config file (optional, highest priority)
    Returns:
        Configured logger instance
    """
    if should_skip_config():
        return setup_from_env()
    config_path = find_config_file(explicit_config)
    if config_path and config_path.exists():
        return setup_from_yaml(config_path)
    else:
        return setup_from_env()


# Level format settings
_level_format = os.environ.get("LOGKISS_LEVEL_FORMAT", "5")
try:
    LEVEL_FORMAT = int(_level_format)
except ValueError:
    LEVEL_FORMAT = 5

# Path shortening settings
_path_shorten = os.environ.get("LOGKISS_PATH_SHORTEN", "0")
try:
    PATH_SHORTEN = int(_path_shorten)
except ValueError:
    # Disable if not a number
    PATH_SHORTEN = 0


@dataclass
class ColorConfig:
    """Data class to hold color settings"""

    color: Optional[str] = None
    background: Optional[str] = None
    style: Optional[str] = None


class Colors:
    """ANSI escape sequences"""

    RESET = "\033[0m"
    BOLD = "\033[1m"
    DIM = "\033[2m"
    ITALIC = "\033[3m"
    UNDERLINE = "\033[4m"

    REVERSE = "\033[7m"
    HIDDEN = "\033[8m"
    STRIKE = "\033[9m"

    # Foreground colors
    BLACK = "\033[30m"
    RED = "\033[31m"
    GREEN = "\033[32m"
    YELLOW = "\033[33m"
    BLUE = "\033[34m"
    MAGENTA = "\033[35m"
    CYAN = "\033[36m"
    WHITE = "\033[37m"

    # Background colors
    BG_BLACK = "\033[40m"
    BG_RED = "\033[41m"
    BG_GREEN = "\033[42m"
    BG_YELLOW = "\033[43m"
    BG_BLUE = "\033[44m"
    BG_MAGENTA = "\033[45m"
    BG_CYAN = "\033[46m"
    BG_WHITE = "\033[47m"

    # Bright foreground colors
    BRIGHT_BLACK = "\033[90m"
    BRIGHT_RED = "\033[91m"
    BRIGHT_GREEN = "\033[92m"
    BRIGHT_YELLOW = "\033[93m"
    BRIGHT_BLUE = "\033[94m"
    BRIGHT_MAGENTA = "\033[95m"
    BRIGHT_CYAN = "\033[96m"
    BRIGHT_WHITE = "\033[97m"

    # Bright background colors
    BG_BRIGHT_BLACK = "\033[100m"
    BG_BRIGHT_RED = "\033[101m"
    BG_BRIGHT_GREEN = "\033[102m"
    BG_BRIGHT_YELLOW = "\033[103m"
    BG_BRIGHT_BLUE = "\033[104m"
    BG_BRIGHT_MAGENTA = "\033[105m"
    BG_BRIGHT_CYAN = "\033[106m"
    BG_BRIGHT_WHITE = "\033[107m"

    @classmethod
    def get_color(cls, name: str) -> str:
        """Get ANSI escape sequence from color name"""
        if not name:
            return ""

        # Add prefix for background colors
        if name.startswith("bg_"):
            name = f"BG_{name[3:].upper()}"
        else:
            name = name.upper()

        return getattr(cls, name, "")


class ColorManager:
    """Class to manage color settings"""

    def __init__(self, config_path: Optional[Union[str, Path]] = None):
        """
        Args:
            config_path: Path to color configuration file
        """
        self.config_path = config_path
        self._config = None  # 内部設定を保持するプライベート変数
        self._external_config = False  # 外部から設定が適用されたかどうか
        # 初期化時にファイルから設定を読み込む
        self._config = self._load_config()

    @property
    def config(self) -> Dict[str, Any]:
        """色設定を取得する"""
        if self._config is None:
            self._config = self._load_default_config()
        return self._config

    @config.setter
    def config(self, value: Dict[str, Any]) -> None:
        """色設定を設定する"""
        import copy
        
        # 完全に置き換えるためにディープコピーを使用
        self._config = copy.deepcopy(value)
        self._external_config = True
        
    def _load_default_config(self) -> Dict[str, Any]:
        """デフォルトの色設定を読み込む"""
        # Default color settings
        return {
            "levels": {
                "DEBUG": {"fg": "blue"},
                "INFO": {"fg": "white"},
                "WARNING": {"fg": "black", "bg": "yellow"},  # 黄色地に黒字に変更
                "ERROR": {"fg": "black", "bg": "red"},
                "CRITICAL": {"fg": "black", "bg": "bright_red", "style": "bold"},
            },
            "elements": {
                "timestamp": {"fg": "white"},
                "filename": {"fg": "cyan"},
                "message": {
                    "DEBUG": {"fg": "blue"},
                    "INFO": {"fg": "white"},
                    "WARNING": {"fg": "black", "bg": "yellow"},  # 黄色地に黒字に変更
                    "ERROR": {"fg": "black", "bg": "red"},
                    "CRITICAL": {"fg": "black", "bg": "bright_red", "style": "bold"},
                },
            },
        }
    
    def _load_config(self) -> Dict[str, Any]:
        """Load color settings from file"""
        # 外部から設定が適用されている場合は、その設定を使用
        if hasattr(self, '_external_config') and self._external_config and self._config is not None:
            return self._config
            
        # デフォルト設定を取得
        default_config = self._load_default_config()

        # Load configuration from file if available
        if self.config_path:
            try:
                with open(self.config_path, "r", encoding='utf-8') as f:
                    config = safe_load(f)
                    if config:
                        # 設定ファイルの内容を優先して適用（デフォルト設定は使用しない）
                        return config
            except (FileNotFoundError, YAMLError, TypeError):
                # ファイルが存在しない、読み込めない、または無効なYAMLの場合
                return default_config
        return default_config

    def get_level_color(self, level: Union[int, str]) -> Dict[str, Any]:
        """Get color settings for a log level"""
        if isinstance(level, int):
            level_name = logging.getLevelName(level)
        else:
            level_name = level
        return self.config["levels"].get(level_name, {})

    def get_message_color(self, level: Union[int, str]) -> Dict[str, Any]:
        """Get color settings for a log message"""
        if isinstance(level, int):
            level_name = logging.getLevelName(level)
        else:
            level_name = level
        return self.config["elements"]["message"].get(level_name, {})

    def get_element_color(self, element: str) -> Dict[str, Any]:
        """Get color settings for a log element"""
        return self.config["elements"].get(element, {})

    def apply_color(self, text: str, config: Dict[str, Any]) -> str:
        """Apply color settings to text"""
        if not config:
            return text

        # Generate ANSI escape sequence
        codes = []

        # Foreground color
        if "fg" in config:
            codes.append(getattr(Colors, config["fg"].upper(), ""))

        # Background color
        if "bg" in config:
            codes.append(getattr(Colors, f"BG_{config['bg'].upper()}", ""))

        # Style
        if "style" in config:
            codes.append(getattr(Colors, config["style"].upper(), ""))

        # Apply ANSI escape sequence
        return "".join(codes) + text + Colors.RESET

    def colorize_level(self, levelname: str, levelno: Optional[int] = None) -> str:
        """Colorize log level name"""
        if levelno is not None:
            level_config = self.get_level_color(levelno)
        else:
            level = logging.getLevelName(levelname)
            level_config = self.get_level_color(level)
        return self.apply_color(levelname, level_config)

    def colorize_filename(self, filename: str) -> str:
        """Colorize filename"""
        filename_config = self.get_element_color("filename")
        return self.apply_color(filename, filename_config)

    def colorize_timestamp(self, timestamp: str) -> str:
        """Colorize timestamp"""
        timestamp_config = self.get_element_color("timestamp")
        return self.apply_color(timestamp, timestamp_config)

    def colorize_message(self, message: str, level: int) -> str:
        """Colorize log message"""
        message_config = self.get_message_color(level)
        return self.apply_color(message, message_config)


class PathShortenerFilter(Filter):
    """Filter to shorten paths in log messages

    Shortens long paths to ".../<last_n_components>" format.
    Example: "/very/long/path/to/module.py" -> ".../to/module.py"

    Controlled by environment variable LOGKISS_PATH_SHORTEN:
    - 0 or invalid value: Disable path shortening
    - Positive integer: Show last n components
    """

    def __init__(self):
        super().__init__()

    def filter(self, record):
        if PATH_SHORTEN > 0:
            # Split path into components
            components = record.pathname.split("/")

            # Get last n components
            if len(components) > PATH_SHORTEN:
                shortened = "/".join(["..."] + components[-PATH_SHORTEN:])
                record.filename = shortened

        return True


class ColoredFormatter(Formatter):
    """Formatter that applies colors to log messages based on their level.

    This formatter extends the standard logging.Formatter to add color
    to log messages based on their level. Colors can be customized through
    a configuration file.

    To disable colors, set use_color=False when creating the formatter:
        formatter = ColoredFormatter(use_color=False)

    Note:
        When using KissConsoleHandler, a ColoredFormatter is automatically
        created with use_color set based on the output stream. To disable
        colors with KissConsoleHandler, you need to create a ColoredFormatter
        with use_color=False and set it explicitly:
            handler = KissConsoleHandler()
            formatter = ColoredFormatter(use_color=False)
            handler.setFormatter(formatter)

    Environment Variables:
        The following environment variables can be used to control coloring:
        - LOGKISS_DISABLE_COLOR: Disable colors (values: 1, true, yes)
        - NO_COLOR: Industry standard to disable colors (any value)
        These environment variables override the use_color parameter.

        The following environment variable can be used to control level name formatting:
        - LOGKISS_LEVEL_FORMAT: Specify the length of level names (value: integer, default: 5)
          For example, LOGKISS_LEVEL_FORMAT=5 will adjust all level names to be 5 characters.
          WARNING is specially shortened to "WARN".
          Level names longer than the specified length will be truncated,
          and shorter ones will be padded with spaces.

    Args:
        fmt: Format string for log messages. Default is
             '%(asctime)s %(levelname)s | %(filename)s: %(lineno)d | %(message)s'
        datefmt: Date format string. Default is None (ISO8601 format).
        style: Format style ('%', '{', '$'). Default is '%'.
        validate: Whether to validate format string. Default is True.
        color_config: Path to color configuration file. Default is None.
        use_color: Whether to apply colors to log messages. Default is True.
    """

    def __init__(
        self,
        fmt: Optional[str] = None,
        datefmt: Optional[str] = None,
        style: str = "%",
        validate: bool = True,
        color_config: Optional[Union[str, Path]] = None,
        use_color: bool = True,
        format_str: Optional[str] = None,
        **kwargs
    ):
        """
        Args:
            fmt: Format string
            datefmt: Date format string
            style: Format style ('%', '{', '$')
            validate: Validate format string
            color_config: Path to color configuration file
            use_color: Apply colors to log messages
            format_str: 別名のフォーマット文字列（後方互換性用）
            **kwargs: 後方互換性のための追加パラメータ（例：'format'）
        """
        # 後方互換性のために'format'パラメータをサポート
        if 'format' in kwargs and format_str is None:
            format_str = kwargs.pop('format')
        if fmt is None and format_str is not None:
            fmt = format_str
        elif fmt is None:
            fmt = "%(asctime)s %(levelname)s | %(filename)s: %(lineno)d | %(message)s"
        # Python 3.7 compatibility: validate parameter was added in Python 3.8
        if sys.version_info >= (3, 8):
            super().__init__(fmt, datefmt, style, validate)
        else:
            super().__init__(fmt, datefmt, style)
        self.color_manager = ColorManager(color_config)

        # Check if color should be disabled via environment variables
        # 1. LOGKISS_DISABLE_COLOR: Specific to this library
        disable_color = os.environ.get("LOGKISS_DISABLE_COLOR", "").lower() in ("1", "true", "yes")
        # 2. NO_COLOR: Industry standard (https://no-color.org/)
        # Any value (or even empty string) means disable color
        no_color = "NO_COLOR" in os.environ

        # Environment variables take precedence over the use_color parameter
        if disable_color or no_color:
            self.use_color = False
        else:
            self.use_color = use_color

    def format(self, record: LogRecord) -> str:
        """Format log record with colors"""
        # Save original levelname and level number
        orig_levelname = record.levelname
        levelno = record.levelno

        # Format level name based on LEVEL_FORMAT
        if LEVEL_FORMAT > 0:
            # Special case for WARNING -> WARN
            if orig_levelname == "WARNING":
                display_levelname = "WARN"
            else:
                display_levelname = orig_levelname

            # Truncate or pad level name
            if len(display_levelname) > LEVEL_FORMAT:
                # Truncate
                display_levelname = display_levelname[:LEVEL_FORMAT]
            elif len(display_levelname) < LEVEL_FORMAT:
                # Pad
                display_levelname = display_levelname.ljust(LEVEL_FORMAT)

            # Replace levelname with formatted version
            record.levelname = display_levelname

        # Apply colors
        if self.use_color:
            # Use original level for color lookup, but apply to formatted level name
            record.levelname = self.color_manager.colorize_level(record.levelname, levelno)

            record.filename = self.color_manager.colorize_filename(record.filename)
            record.asctime = self.color_manager.colorize_timestamp(self.formatTime(record, self.datefmt))
            record.message = self.color_manager.colorize_message(record.getMessage(), levelno)
        else:
            record.message = record.getMessage()

        # Format record
        return Formatter.format(self, record)


class KissConsoleHandler(StreamHandler):
    """Handler that outputs colored log messages to the console.

    This handler extends the standard logging.StreamHandler to add color
    to log messages based on their level. Colors can be customized through
    a configuration file.

    By default, colors are enabled when outputting to sys.stderr or sys.stdout.
    To disable colors, you need to create a ColoredFormatter with use_color=False
    and set it explicitly:
        handler = KissConsoleHandler()
        formatter = ColoredFormatter(use_color=False)
        handler.setFormatter(formatter)

    Environment Variables:
        The following environment variables can be used to control coloring:
        - LOGKISS_DISABLE_COLOR: Disable colors (values: 1, true, yes)
        - NO_COLOR: Industry standard to disable colors (any value)
        These environment variables override the use_color parameter of the formatter.

    Args:
        stream: Output stream. Default is sys.stderr.
        color_config: Path to color configuration file. Default is None.
    """

    def __init__(self, stream: Optional[TextIO] = None, color_config: Optional[Union[str, Path]] = None):
        """
        Args:
            stream: Output stream
            color_config: Path to color configuration file
        """
        # Default to sys.stderr
        if stream is None:
            stream = sys.stderr

        super().__init__(stream)

        # Check environment variables for disabling color
        # 1. LOGKISS_DISABLE_COLOR: Specific to this library
        disable_color = os.environ.get("LOGKISS_DISABLE_COLOR", "").lower() in ("1", "true", "yes")
        # 2. NO_COLOR: Industry standard (https://no-color.org/)
        # Any value (or even empty string) means disable color
        no_color = "NO_COLOR" in os.environ

        # Apply colors if not disabled by env vars and outputting to sys.stderr or sys.stdout
        use_color = not (disable_color or no_color) and (stream is None or stream is sys.stderr or stream is sys.stdout)

        self.formatter = ColoredFormatter(color_config=color_config, use_color=use_color)
        self.setFormatter(self.formatter)

        # Add path shortening filter
        self.addFilter(PathShortenerFilter())

    def format(self, record: LogRecord) -> str:
        """Format log record"""
        # Set default formatter if not set
        if self.formatter is None:
            self.formatter = ColoredFormatter()
        return self.formatter.format(record)

    def emit(self, record: LogRecord) -> None:
        """Output log record"""
        try:
            msg = self.format(record)
            stream = self.stream
            # if exception information is present, it's formatted as text and appended to msg
            stream.write(msg + self.terminator)
            self.flush()
        except (ValueError, TypeError, IOError):
            # 書き込みエラーや型変換エラーの場合
            self.handleError(record)


class KissLogger(logging.Logger):
    """Logger that uses colored output by default"""

    def __init__(self, name: str):
        """Initialize the logger with the specified name"""
        super().__init__(name)
        self.setLevel(logging.WARNING)  # Set default level to WARNING

    def setLevel(self, level: int) -> None:
        """Set the logging level for both logger and handlers"""
        super().setLevel(level)  # Call parent's setLevel
        # Update handler levels
        for handler in self.handlers:
            handler.setLevel(level)

    def makeRecord(
        self,
        name: str,
        level: int,
        fn: str,
        lno: int,
        msg: str,
        args: tuple,
        exc_info: Optional[bool],
        func: Optional[str] = None,
        extra: Optional[Dict[str, Any]] = None,
        sinfo: Optional[str] = None,
    ) -> LogRecord:
        """Create a LogRecord with the given arguments"""
        # Get caller information from extra
        if extra is not None:
            if "_filename" in extra:
                fn = extra["_filename"]
            if "_lineno" in extra:
                lno = extra["_lineno"]

        # Shorten path if enabled
        if os.environ.get("LOGKISS_PATH_SHORTEN", "1").lower() in ("1", "true", "yes"):
            # Use only filename
            fn = os.path.basename(fn)

        # Create LogRecord
        record = super().makeRecord(name, level, fn, lno, msg, args, exc_info, func, extra, sinfo)
        return record

    def reload_config(self) -> None:
        """Reload configuration from the original source.

        This method reloads the logger configuration from the original YAML file
        if it was configured using yaml_config, or from environment variables
        if it was configured using dictConfig with environment variables.

        Raises:
            ValueError: If the logger was not configured with a config_path attribute.
        """
        # Check if config path is available
        if hasattr(self, "config_path"):
            # Remove existing handlers
            for handler in self.handlers[:]:
                self.removeHandler(handler)

            # Reload from YAML
            from logkiss import yaml_config
            yaml_config(self.config_path)
        else:
            # Reload from environment variables
            from logkiss.config import _auto_config_from_env
            _auto_config_from_env()


# Helper function to use a standard ConsoleHandler
def use_console_handler(logger: Optional[logging.Logger] = None) -> None:
    """Configure the logger to use a standard StreamHandler instead of KissConsoleHandler.

    This function removes any existing KissConsoleHandler from the specified logger
    and adds a standard StreamHandler with a basic formatter. This is useful when
    you want to disable the colored output and use a simple console handler.

    Args:
        logger: Logger to configure. Default is None (root logger).

    Returns:
        None

    Example:
        >>> import logkiss as logging
        >>> logger = logging.getLogger(__name__)
        >>> logging.use_console_handler(logger)
        >>> logger.info('This message will be displayed without color')
    """
    # Get the root logger if no logger is specified
    if logger is None:
        logger = logging.getLogger()

    # Remove KissConsoleHandler
    for handler in logger.handlers[:]:
        if isinstance(handler, KissConsoleHandler):
            logger.removeHandler(handler)

    # Add standard ConsoleHandler
    handler = StreamHandler()
    handler.setFormatter(
        Formatter(fmt="%(asctime)s,%(msecs)03d %(levelname)-5s | %(filename)s:%(lineno)3d | %(message)s", datefmt="%Y-%m-%d %H:%M:%S")
    )
    logger.addHandler(handler)


def setup_from_yaml(config_path: Union[str, Path]) -> logging.Logger:
    """Set up logging configuration from a YAML file.

    Args:
        config_path: Path to YAML configuration file

    Returns:
        Configured logger instance

    Raises:
        ValueError: If config file does not exist
        yaml.YAMLError: If config file is invalid YAML
    """
    config_path = Path(config_path)
    if not config_path.exists():
        raise ValueError(f"Configuration file not found: {config_path}")

    with open(config_path, encoding='utf-8') as f:
        config = safe_load(f)
    print("[DEBUG] loaded config:", config)

    # Get root logger
    logger = logging.getLogger()

    # Configure formatters
    formatters = {}
    for name, formatter_config in config.get("formatters", {}).items():
        formatters[name] = ColoredFormatter(**formatter_config)

    # Default to console handler if none specified
    if not config.get("handlers"):
        config["handlers"] = {"console": {"class": "StreamHandler"}}
    # Default root handlers if none specified
    root_conf = config.setdefault("root", {})
    if not root_conf.get("handlers"):
        root_conf["handlers"] = ["console"]

    # Configure handlers
    print("[DEBUG] handlers section:", config.get("handlers", {}))
    # Remove existing logkiss handlers, preserve external handlers (e.g., caplog)
    for h in logger.handlers[:]:
        if isinstance(h, (KissConsoleHandler, FileHandler, RotatingFileHandler, TimedRotatingFileHandler)):
            logger.removeHandler(h)

    handler_objs = {}
    for name, handler_config in config.get("handlers", {}).items():
        handler_config = handler_config.copy()  # dictの副作用防止
        handler_class = handler_config.pop("class")
        formatter = handler_config.pop("formatter", None)
        level = handler_config.pop("level", None)

        # Create handler instance
        if handler_class in ("logging.StreamHandler", "StreamHandler", "KissConsoleHandler"):
            handler = KissConsoleHandler()
        elif handler_class in ("logging.FileHandler", "FileHandler"):
            try:
                print("[DEBUG] FileHandler config:", handler_config)
                handler = FileHandler(**handler_config)
            except Exception as e:
                print("[ERROR] FileHandler creation failed:", e)
                raise
        elif handler_class in ("logging.handlers.RotatingFileHandler", "RotatingFileHandler"):
            try:
                print("[DEBUG] RotatingFileHandler config:", handler_config)
                handler = RotatingFileHandler(**handler_config)
            except Exception as e:
                print("[ERROR] RotatingFileHandler creation failed:", e)
                raise
        elif handler_class in ("logging.TimedRotatingFileHandler", "TimedRotatingFileHandler"):
            try:
                print("[DEBUG] TimedRotatingFileHandler config:", handler_config)
                handler = TimedRotatingFileHandler(**handler_config)
            except Exception as e:
                print("[ERROR] TimedRotatingFileHandler creation failed:", e)
                raise
        else:
            continue

        # Set formatter if specified
        if formatter and formatter in formatters:
            handler.setFormatter(formatters[formatter])

        # Set level if specified
        if level:
            handler.setLevel(getattr(logging, level.upper()))

        handler_objs[name] = handler
    # root["handlers"]に従ってのみaddHandler
    for hname in config.get("root", {}).get("handlers", []):
        if hname in handler_objs:
            logger.addHandler(handler_objs[hname])

    # Configure root logger
    root_config = config.get("root", {})

    # Check environment variables first
    env_level = os.environ.get("LOGKISS_LEVEL")
    if env_level:
        logger.setLevel(getattr(logging, env_level.upper()))
    elif "level" in root_config:
        logger.setLevel(getattr(logging, root_config["level"]))
    else:
        # Default root level to WARNING if not specified
        logger.setLevel(logging.WARNING)

    print("[DEBUG] logger.handlers:", logger.handlers)
    return logger


def setup_from_env() -> logging.Logger:
    """Set up logging configuration from environment variables.

    Environment variables:
        LOGKISS_LEVEL: Log level (DEBUG, INFO, WARNING, ERROR, CRITICAL)
        LOGKISS_FORMAT: Log format string
        LOGKISS_DISABLE_COLOR: Disable colored output if 'true'
        NO_COLOR: Industry standard to disable colors (any value)

    Returns:
        Configured logger instance
    """
    # Get root logger
    logger = logging.getLogger()

    # Configure level
    level = os.environ.get("LOGKISS_LEVEL", "WARNING")
    logger.setLevel(getattr(logging, level.upper()))

    # Create a console handler
    handler = StreamHandler(sys.stderr)

    # Configure formatter
    fmt = os.environ.get("LOGKISS_FORMAT", "%(asctime)s,%(msecs)03d %(levelname)-5s | %(filename)s:%(lineno)3d | %(message)s")
    datefmt = os.environ.get("LOGKISS_DATEFMT", "%Y-%m-%d %H:%M:%S")

    # Determine color usage based on environment variables
    use_color = True  # Default is to use color
    
    # 1. LOGKISS_DISABLE_COLOR: Specific to this library
    if os.environ.get("LOGKISS_DISABLE_COLOR", "").lower() in ("1", "true", "yes"):
        use_color = False
        
    # 2. NO_COLOR: Industry standard (https://no-color.org/)
    # Any value (or even empty string) means disable color
    if "NO_COLOR" in os.environ:
        use_color = False

    # Create formatter with color settings
    formatter = ColoredFormatter(fmt=fmt, datefmt=datefmt, use_color=use_color)

    # Set the formatter on the handler
    handler.setFormatter(formatter)

    # Add handler to logger
    logger.addHandler(handler)

    return logger



"""logkiss - Keep It Simple and Stupid Logger.

Copyright (c) 2025 Taka Suzuki
SPDX-License-Identifier: MIT
See LICENSE for details.

A simple and colorful Python logging library.
"""

#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import logging
import logging.handlers
from pathlib import Path
from typing import Optional, Union, Dict, Any

# Set debug mode
DEBUG_MODE = os.environ.get("LOGKISS_DEBUG", "0").lower() in ("1", "true", "yes")

# Import logkiss module classes
from .logkiss import (
    KissLogger,
    KissConsoleHandler,
    ColoredFormatter,
)

# Import config module
from .config import dictConfig, fileConfig, yaml_config

# Import base handler only
from .handlers import BaseHandler

# プロキシクラスと遅延インポート機能
# クラウドサービス依存関係を実際に使用するまで読み込まない


# AWS CloudWatchハンドラー用プロキシクラス
class AWSCloudWatchHandler:
    """AWS CloudWatch Logs対応のロギングハンドラー

    実際のAWS SDK (boto3)の読み込みは初めて使用されるときまで遅延されます。
    """

    def __new__(cls, *args, **kwargs):
        # 遅延インポート
        try:
            from .handlers import AWSCloudWatchHandler as RealHandler

            # 実際のクラスのインスタンスを返す
            return RealHandler(*args, **kwargs)
        except ImportError as exc:
            raise ImportError(
                "AWSCloudWatchHandler requires boto3 package. " "Install it with: pip install 'logkiss[aws]' or pip install boto3"
            ) from exc


# GCP Cloud Loggingハンドラー用プロキシクラス
class GCloudLoggingHandler:
    """Google Cloud Logging対応のロギングハンドラー

    実際のGoogle Cloud SDK (google-cloud-logging)の読み込みは初めて使用されるときまで遅延されます。
    """

    def __new__(cls, *args, **kwargs):
        # 遅延インポート
        try:
            from .handler_gcp import GCloudLoggingHandler as RealHandler

            # 実際のクラスのインスタンスを返す
            return RealHandler(*args, **kwargs)
        except ImportError as exc:
            raise ImportError(
                "GCloudLoggingHandler requires google-cloud-logging package. "
                "Install it with: pip install 'logkiss[gcp]' or pip install google-cloud-logging"
            ) from exc


# GCPロギング設定関数の遅延インポート版
def setup_gcp_logging(*args, **kwargs):
    """Google Cloud Loggingの設定を行う（遅延ロード）

    実際のGoogle Cloud SDK (google-cloud-logging)の読み込みは初めて使用されるときまで遅延されます。
    """
    try:
        from .handler_gcp import setup_gcp_logging as real_setup

        return real_setup(*args, **kwargs)
    except ImportError as exc:
        raise ImportError(
            "setup_gcp_logging requires google-cloud-logging package. "
            "Install it with: pip install 'logkiss[gcp]' or pip install google-cloud-logging"
        ) from exc


# Try to import Qt handler if available
try:
    from .handler_qt import QtTextEditHandler, QT_AVAILABLE
except ImportError:
    QT_AVAILABLE = False

# Import standard logging module functions
from logging import (
    BASIC_FORMAT,
    CRITICAL,
    DEBUG,
    ERROR,
    FATAL,
    INFO,
    NOTSET,
    WARN,
    WARNING,
    addLevelName,
    basicConfig,
    critical,
    debug,
    disable,
    error,
    exception,
    fatal,
    getLevelName,
    getLogger as _getLogger,
    getLoggerClass,
    info,
    log,
    makeLogRecord,
    setLoggerClass,
    warn,
    warning,
    getLogRecordFactory,
    setLogRecordFactory,
)

# logkissとしてgetLoggerを標準logging.getLoggerで公開
getLogger = logging.getLogger


def init_logging(*args, **kwargs):
    """
    [DEPRECATED] For backward compatibility only. Does nothing.
    Use standard logging.basicConfig or logging.getLogger instead.
    """
    import warnings

    warnings.warn(
        "logkiss.init_logging() is deprecated. Use standard logging.basicConfig or logging.getLogger instead.", DeprecationWarning, stacklevel=2
    )
    return logging.getLogger()


# Define __all__
__all__ = [
    "BASIC_FORMAT",
    "BufferingFormatter",
    "CRITICAL",
    "DEBUG",
    "ERROR",
    "FATAL",
    "FileHandler",
    "Filter",
    "Formatter",
    "Handler",
    "INFO",
    "LogRecord",
    "Logger",
    "LoggerAdapter",
    "NOTSET",
    "NullHandler",
    "StreamHandler",
    "WARN",
    "WARNING",
    "addLevelName",
    "basicConfig",
    "critical",
    "debug",
    "disable",
    "error",
    "exception",
    "fatal",
    "getLevelName",
    "getLogger",
    "getLoggerClass",
    "info",
    "log",
    "makeLogRecord",
    "setLoggerClass",
    "warn",
    "warning",
    # ハンドラー
    "BaseHandler",
    "AWSCloudWatchHandler",
    "GCloudLoggingHandler",
    "setup_gcp_logging",
    # Qt handler
    "QtTextEditHandler",
    "QT_AVAILABLE",
    # 設定
    "dictConfig",
    "fileConfig",
    "yaml_config",
]

# グローバル設定
# Inherit standard logging module functions
BASIC_FORMAT = logging.BASIC_FORMAT
CRITICAL = logging.CRITICAL
FATAL = logging.FATAL
ERROR = logging.ERROR
WARNING = logging.WARNING
WARN = logging.WARN
INFO = logging.INFO
DEBUG = logging.DEBUG
NOTSET = logging.NOTSET

# Inherit logging module functions
disable = logging.disable
shutdown = logging.shutdown
addLevelName = logging.addLevelName
getLevelName = logging.getLevelName
makeLogRecord = logging.makeLogRecord


def getLoggerClass() -> type:
    """Get current logger class"""
    return logging.getLoggerClass()


def setLoggerClass(cls: type) -> None:
    """Set logger class"""
    logging.setLoggerClass(cls)


# Inherit standard logging module classes
BufferingFormatter = logging.BufferingFormatter
Filter = logging.Filter
Formatter = logging.Formatter
Handler = logging.Handler
LogRecord = logging.LogRecord
Logger = logging.Logger
LoggerAdapter = logging.LoggerAdapter
Manager = logging.Manager
PlaceHolder = logging.PlaceHolder
StreamHandler = logging.StreamHandler
FileHandler = logging.FileHandler
NullHandler = logging.NullHandler
RotatingFileHandler = logging.handlers.RotatingFileHandler
TimedRotatingFileHandler = logging.handlers.TimedRotatingFileHandler

# Inherit other variables
raiseExceptions = logging.raiseExceptions


def use_console_handler(logger: Optional[logging.Logger] = None) -> None:
    """Use standard ConsoleHandler

    .. deprecated:: 2.2.0
       This function is deprecated. Use standard logging methods instead:

       .. code-block:: python

          # Remove existing handlers and add a standard one
          logger.handlers.clear()
          logger.addHandler(logging.StreamHandler())
    """
    import warnings

    warnings.warn(
        "use_console_handler is deprecated. Use standard logging methods instead: "
        "logger.handlers.clear() and logger.addHandler(logging.StreamHandler())",
        DeprecationWarning,
        stacklevel=2,
    )

    if logger is None:
        logger = logging.getLogger()

    # Remove KissConsoleHandler
    for handler in logger.handlers[:]:
        if isinstance(handler, KissConsoleHandler):
            logger.removeHandler(handler)

    # Add standard ConsoleHandler
    handler = logging.StreamHandler()
    handler.setFormatter(
        logging.Formatter(fmt="%(asctime)s,%(msecs)03d %(levelname)-5s | %(filename)s:%(lineno)3d | %(message)s", datefmt="%Y-%m-%d %H:%M:%S")
    )
    logger.addHandler(handler)


# Version information
__version__ = "2.3.1"

# --- logkiss default handler initialization ---
# --- logkiss default handler initialization ---
root_logger = logging.getLogger()
root_logger.handlers.clear()  # 既存のハンドラを全て除去
handler = KissConsoleHandler()

# --- サブロガーにハンドラがある場合はルートで出力しないフィルタ ---
class _SkipIfLoggerHasHandlers(logging.Filter):
    def filter(self, record):
        logger = logging.getLogger(record.name)
        # サブロガーで、ルート以外でハンドラが1つ以上あればルートで出力しない
        if logger is not root_logger and logger.handlers:
            return False
        return True
handler.addFilter(_SkipIfLoggerHasHandlers())
# -------------------------------------------------------------

root_logger.addHandler(handler)
root_logger.propagate = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
AWS CloudWatch Logs Handler for logkiss.

This module provides a handler for sending logs to AWS CloudWatch Logs.
It uses the boto3 library for AWS integration.

Copyright (c) 2025 Taka Suzuki
SPDX-License-Identifier: MIT
"""

import logging
import os
import json
import time
from typing import Dict, Any, Optional, Union
from datetime import datetime

try:
    import boto3

    AWS_AVAILABLE = True
except ImportError:
    AWS_AVAILABLE = False


class AWSCloudWatchHandler(logging.Handler):
    """AWS CloudWatch Logs handler for logkiss.

    This handler uses boto3 to send logs to AWS CloudWatch Logs.
    It provides a simple interface for configuring and using the handler.

    Args:
        log_group (str): The name of the CloudWatch Logs log group.
        log_stream (str, optional): The name of the CloudWatch Logs log stream.
            If not provided, a default stream name will be generated.
        aws_region (str, optional): AWS region name. If not provided, it will be
            determined from the environment or AWS configuration.
        aws_access_key_id (str, optional): AWS access key ID. If not provided,
            it will be determined from the environment or AWS configuration.
        aws_secret_access_key (str, optional): AWS secret access key. If not provided,
            it will be determined from the environment or AWS configuration.
        batch_size (int, optional): Maximum number of log events to send in a single batch.
            Defaults to 10000 (AWS CloudWatch Logs limit).
    """

    def __init__(
        self,
        log_group: str,
        log_stream: Optional[str] = None,
        aws_region: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        batch_size: int = 10000,
    ) -> None:
        """Initialize the handler."""
        if not AWS_AVAILABLE:
            raise ImportError("boto3 is required to use AWSCloudWatchHandler. " "Please install it with: pip install boto3")

        super().__init__()

        self.log_group = log_group
        self.log_stream = log_stream or f"logkiss-{datetime.now().strftime('%Y-%m-%d-%H-%M-%S')}"
        self.batch_size = min(batch_size, 10000)  # AWS limit

        # Initialize boto3 client
        self.client = boto3.client(
            "logs",
            region_name=aws_region,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
        )

        # Ensure log group exists
        self._create_log_group()

        # Create log stream
        self._create_log_stream()

        # Initialize sequence token
        self.sequence_token = None

        # Set formatter
        formatter = logging.Formatter("%(message)s")
        self.setFormatter(formatter)

    def _create_log_group(self) -> None:
        """Create the log group if it doesn't exist."""
        try:
            self.client.create_log_group(logGroupName=self.log_group)
        except self.client.exceptions.ResourceAlreadyExistsException:
            pass

    def _create_log_stream(self) -> None:
        """Create the log stream if it doesn't exist."""
        try:
            self.client.create_log_stream(logGroupName=self.log_group, logStreamName=self.log_stream)
        except self.client.exceptions.ResourceAlreadyExistsException:
            pass

    def emit(self, record: logging.LogRecord) -> None:
        """Emit a log record.

        Args:
            record (logging.LogRecord): The log record to emit.
        """
        # Prepare the log event
        log_event = {
            "timestamp": int(record.created * 1000),
            "message": self.format(record),
        }

        # Add structured logging if extra fields are present
        if hasattr(record, "extra") and isinstance(record.extra, dict):
            try:
                # Convert the message to JSON format
                log_event["message"] = json.dumps({"message": log_event["message"], "extra": record.extra})
            except (TypeError, ValueError):
                # Fallback to string representation if JSON conversion fails
                log_event["message"] = f"{log_event['message']} {str(record.extra)}"

        # Send the log event
        self._put_log_events([log_event])

    def _put_log_events(self, log_events: list) -> None:
        """Send log events to CloudWatch Logs.

        Args:
            log_events (list): List of log events to send.
        """
        kwargs = {"logGroupName": self.log_group, "logStreamName": self.log_stream, "logEvents": log_events}

        if self.sequence_token:
            kwargs["sequenceToken"] = self.sequence_token

        try:
            response = self.client.put_log_events(**kwargs)
            self.sequence_token = response.get("nextSequenceToken")
        except (self.client.exceptions.InvalidSequenceTokenException, self.client.exceptions.DataAlreadyAcceptedException) as e:
            # Handle sequence token issues
            if hasattr(e, "response"):
                self.sequence_token = e.response.get("expectedSequenceToken")
                if self.sequence_token:
                    # Retry with correct sequence token
                    kwargs["sequenceToken"] = self.sequence_token
                    response = self.client.put_log_events(**kwargs)
                    self.sequence_token = response.get("nextSequenceToken")

    def close(self) -> None:
        """Close the handler."""
        self.client = None
        super().close()


def setup_aws_logging(
    log_group: str,
    log_stream: Optional[str] = None,
    aws_region: Optional[str] = None,
    level: Union[int, str] = logging.INFO,
) -> AWSCloudWatchHandler:
    """AWS CloudWatch Logsの設定を行います。

    Args:
        log_group: CloudWatch Logsのロググループ名
        log_stream: CloudWatch Logsのログストリーム名（省略可）
        aws_region: AWSリージョン名（省略可）
        level: ログレベル

    Returns:
        AWSCloudWatchHandler: 設定されたハンドラー
    """
    # ルートロガーを取得
    root_logger = logging.getLogger()

    # AWSCloudWatchHandlerを作成
    handler = AWSCloudWatchHandler(
        log_group=log_group,
        log_stream=log_stream,
        aws_region=aws_region,
    )

    # ログレベルを設定
    handler.setLevel(level)

    # ルートロガーにハンドラーを追加
    root_logger.addHandler(handler)

    return handler


def create_handler(
    log_group: str,
    log_stream: Optional[str] = None,
    aws_region: Optional[str] = None,
) -> AWSCloudWatchHandler:
    """Create an AWSCloudWatchHandler with minimal configuration.

    Args:
        log_group (str): The name of the CloudWatch Logs log group.
        log_stream (str, optional): The name of the CloudWatch Logs log stream.
        aws_region (str, optional): AWS region name.

    Returns:
        AWSCloudWatchHandler: The created handler.
    """
    # Use environment variable if aws_region is not provided
    if aws_region is None:
        aws_region = os.environ.get("AWS_REGION")

    # Create the handler
    return AWSCloudWatchHandler(
        log_group=log_group,
        log_stream=log_stream,
        aws_region=aws_region,
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Qt handler for logkiss.
This module provides a handler for logging to Qt text widgets.
The Qt module is optional and only required if you want to use the QtTextEditHandler.
"""

import logging
import sys
from typing import Optional, Dict, Any, Union

# Try to import Qt modules
QT_AVAILABLE = False
try:
    # Try PyQt5 first
    from PyQt5.QtWidgets import QTextEdit
    from PyQt5.QtGui import QColor, QTextCharFormat, QBrush, QFont
    from PyQt5.QtCore import Qt

    QT_AVAILABLE = True
except ImportError:
    try:
        # Try PySide2 as fallback
        from PySide2.QtWidgets import QTextEdit
        from PySide2.QtGui import QColor, QTextCharFormat, QBrush, QFont
        from PySide2.QtCore import Qt

        QT_AVAILABLE = True
    except ImportError:
        try:
            # Try PySide6 as another fallback
            from PySide6.QtWidgets import QTextEdit
            from PySide6.QtGui import QColor, QTextCharFormat, QBrush, QFont
            from PySide6.QtCore import Qt

            QT_AVAILABLE = True
        except ImportError:
            # Define placeholder classes for type hinting when Qt is not available
            class QTextEdit:
                pass

            class QColor:
                def __init__(self, *args):
                    pass

            class QFont:
                Bold = 75

            QBrush = object
            QTextCharFormat = object
            Qt = object


# Import from logkiss
from . import logkiss


class QtTextEditHandler(logging.Handler):
    """
    Custom logging handler that outputs log messages to a QTextEdit widget with colors.
    Supports light and dark themes.

    This handler requires PyQt5, PySide2, or PySide6 to be installed.
    If none of these modules are available, an ImportError will be raised when
    trying to use the handler.

    Args:
        text_edit: A QTextEdit widget to output logs to
        theme: The theme to use, "light" or "dark", defaults to "light"
        formatter: Optional custom formatter, if None, logkiss.ColoredFormatter is used

    Example:
        >>> import logkiss
        >>> from PyQt5.QtWidgets import QApplication, QTextEdit
        >>> app = QApplication([])
        >>> text_edit = QTextEdit()
        >>> handler = logkiss.QtTextEditHandler(text_edit)
        >>> logger = logkiss.getLogger()
        >>> logger.addHandler(handler)
        >>> logger.info("Hello from Qt!")
    """

    def __init__(self, text_edit: QTextEdit, theme: str = "light", formatter: Optional[logging.Formatter] = None):
        if not QT_AVAILABLE:
            raise ImportError("Qt modules not available. Install PyQt5, PySide2, or PySide6 to use QtTextEditHandler.")

        super().__init__()
        self.text_edit = text_edit
        self.formatter = formatter or logkiss.ColoredFormatter()
        self.theme = theme
        self.set_theme(theme)

    def set_theme(self, theme: str) -> None:
        """
        Set color theme (light or dark)

        Args:
            theme: The theme to use, "light" or "dark"
        """
        self.theme = theme
        if theme == "light":
            self.text_edit.setStyleSheet("background-color: white; color: black;")
            # Colors for light theme (darker colors for better visibility)
            self.level_colors = {
                logging.DEBUG: QColor(0, 130, 130),  # dark cyan
                logging.INFO: QColor(0, 0, 0),  # black
                logging.WARNING: QColor(180, 90, 0),  # dark orange
                logging.ERROR: QColor(200, 0, 0),  # dark red
                logging.CRITICAL: QColor(200, 0, 0),  # dark red (with background)
            }
            self.level_backgrounds = {
                logging.CRITICAL: QColor(255, 210, 210),  # light red background
            }
        else:  # dark theme
            self.text_edit.setStyleSheet("background-color: #2d2d2d; color: #f0f0f0;")
            # Colors for dark theme (brighter colors)
            self.level_colors = {
                logging.DEBUG: QColor(0, 255, 255),  # cyan
                logging.INFO: QColor(255, 255, 255),  # white
                logging.WARNING: QColor(255, 255, 0),  # yellow
                logging.ERROR: QColor(255, 100, 100),  # light red
                logging.CRITICAL: QColor(255, 100, 100),  # light red (with background)
            }
            self.level_backgrounds = {
                logging.CRITICAL: QColor(100, 0, 0),  # dark red background
            }

        # Font styles are the same for both themes
        self.level_styles = {
            logging.WARNING: QFont.Bold,
            logging.ERROR: QFont.Bold,
            logging.CRITICAL: QFont.Bold,
        }

    def emit(self, record: logging.LogRecord) -> None:
        """
        Emit a log record to the QTextEdit widget with color formatting.

        Args:
            record: The log record to emit
        """
        if not QT_AVAILABLE:
            return

        try:
            # Get formatted message
            msg = self.format(record)

            # Create text format
            text_format = QTextCharFormat()
            text_format.setForeground(QBrush(self.level_colors.get(record.levelno, QColor(255, 255, 255))))

            # Apply background color if specified for the level
            if record.levelno in self.level_backgrounds:
                text_format.setBackground(QBrush(self.level_backgrounds[record.levelno]))

            # Apply font style if specified for the level
            if record.levelno in self.level_styles:
                font = text_format.font()
                font.setWeight(self.level_styles[record.levelno])
                text_format.setFont(font)

            # Insert text with formatting
            cursor = self.text_edit.textCursor()
            cursor.movePosition(cursor.End)
            cursor.insertText(msg + "\n", text_format)

            # Auto-scroll to bottom
            self.text_edit.setTextCursor(cursor)
            self.text_edit.ensureCursorVisible()
        except Exception:
            self.handleError(record)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Google Cloud Logging Handler for logkiss.

This module provides a handler for sending logs to Google Cloud Logging.
It uses the official Google Cloud Logging client library.

Copyright (c) 2025 Taka Suzuki
SPDX-License-Identifier: MIT
"""

# Python標準のloggingを明示的に参照するためのエイリアス
import logging as std_logging
import os
from typing import Dict, Any, Optional, Union


# Flag to track if Google Cloud Logging is available
def _check_gcp_available():
    """Google Cloud Loggingが利用可能か確認する関数"""
    try:
        # これらのインポートは実際に使用していないが、可用性チェックに必要
        import google.cloud.logging  # noqa: F401
        import google.cloud.logging_v2.handlers  # noqa: F401

        return True
    except ImportError:
        return False


class GCloudLoggingHandler(std_logging.Handler):
    """Google Cloud Logging handler for logkiss.

    This handler uses the official Google Cloud Logging client library to send logs
    to Google Cloud Logging. It provides a simple interface for configuring and using
    the handler.

    Args:
        project_id (str, optional): Google Cloud Project ID. If not provided, it will be
            determined from the environment.
        credentials (google.auth.credentials.Credentials, optional): Google Cloud credentials.
            If not provided, default credentials will be used.
        labels (Dict[str, str], optional): Labels to add to all log entries.
        log_name (str, optional): Name of the log to write to. Defaults to 'python'.
        resource (google.cloud.logging_v2.resource.Resource, optional): Monitored resource
            to use for logging. If not provided, it will be determined from the environment.
        excluded_loggers (list, optional): List of logger names to exclude from logging.
    """

    def __init__(
        self,
        project_id: Optional[str] = None,
        credentials: Any = None,
        log_name: str = "python",
        labels: Optional[Dict[str, str]] = None,
        resource: Any = None,
        excluded_loggers: Optional[list] = None,
    ) -> None:
        """Initialize the handler.

        Args:
            project_id: Google Cloud project ID. If None, it will be automatically detected.
            credentials: Google Cloud credentials. If None, default credentials will be used.
            log_name: Name of the log to write to.
            labels: Labels to add to all log entries.
            resource: Monitored resource to use for logging.
            excluded_loggers: List of logger names to exclude from logging.

        Raises:
            ImportError: If Google Cloud Logging is not available.
        """
        # 必要なモジュールをインポート - 実際に必要になった時点でインポート
        try:
            from google.cloud import logging
            from google.cloud.logging_v2.handlers import CloudLoggingHandler
        except ImportError as exc:
            raise ImportError(
                "Google Cloud Logging is not available. " "Please install the required dependencies using: " "pip install google-cloud-logging"
            ) from exc

        super().__init__()

        # Initialize Google Cloud Logging clien
        client = logging.Client(project=project_id, credentials=credentials)

        # Create the handler with the specified configuration
        self.handler = CloudLoggingHandler(
            client,
            name=log_name,
            labels=labels,
            resource=resource,
        )

        # Store excluded loggers
        self.excluded_loggers = excluded_loggers or []

        # Formatter for the handler - 標準のloggingを使用
        formatter = std_logging.Formatter("%(message)s")
        self.setFormatter(formatter)

    def emit(self, record: std_logging.LogRecord) -> None:
        """Emit a log record.

        Args:
            record (logging.LogRecord): The log record to emit.
        """
        # Skip excluded loggers
        if record.name in self.excluded_loggers:
            return

        try:
            # Ensure extra is a dict if it exists
            if hasattr(record, "extra") and not isinstance(record.extra, dict):
                record.extra = {"extra": str(record.extra)}

            # Google Cloud Loggingのハンドラーが期待する属性を追加
            # pylint: disable=protected-access
            if not hasattr(record, "_resource"):
                record._resource = None

            # _labels属性を追加
            if not hasattr(record, "_labels"):
                record._labels = {}

            # extraの内容を処理
            if hasattr(record, "extra") and isinstance(record.extra, dict):
                # 特別なキー "json_fields" を使用して構造化ログを設定
                # CloudLoggingHandlerは内部でjson_fieldsをjsonPayloadとして扱う
                record.json_fields = {}
                for key, value in record.extra.items():
                    # 値を文字列に変換（必要な場合）
                    if not isinstance(value, (str, int, float, bool, dict, list, type(None))):
                        value = str(value)
                    record.json_fields[key] = value

                # labelsにも追加（文字列に変換）
                for key, value in record.extra.items():
                    # pylint: disable=protected-access
                    record._labels[key] = str(value) if not isinstance(value, (str, bytes)) else value

            # その他の必要な属性を追加
            # pylint: disable=protected-access
            for attr in ["_trace", "_span_id", "_trace_sampled", "_http_request", "_source_location"]:
                if not hasattr(record, attr):
                    setattr(record, attr, None)

            # Forward to Google Cloud Logging handler
            self.handler.emit(record)
        except (ValueError, TypeError) as err:
            # 値の型エラーなど、よくある例外を具体的に捕捉
            import sys

            print(f"Error in GCloudLoggingHandler.emit (value/type error): {err}", file=sys.stderr)
        except Exception as err:  # pylint: disable=broad-excep
            # ここでは幅広い例外をキャッチする必要がある（ロギングは常に動作すべき）
            import sys

            print(f"Error in GCloudLoggingHandler.emit: {err}", file=sys.stderr)

    def close(self) -> None:
        """Close the handler."""
        try:
            self.handler.close()
        except Exception as err:  # pylint: disable=broad-excep
            # closeメソッドは常に成功すべきなので、幅広い例外をキャッチ
            import sys

            print(f"Error closing GCloudLoggingHandler: {err}", file=sys.stderr)
        finally:
            super().close()


def setup_gcp_logging(
    project_id: Optional[str] = None,
    log_name: str = "python",
    labels: Optional[Dict[str, str]] = None,
    level: Union[int, str] = std_logging.INFO,
    excluded_loggers: Optional[list] = None,
) -> GCloudLoggingHandler:
    """Google Cloud Loggingの設定を行います。

    Args:
        project_id: Google CloudプロジェクトのプロジェクトID。Noneの場合は環境変数から自動検出します。
        log_name: ログ名
        labels: すべてのログエントリに追加するラベル
        level: ログレベル
        excluded_loggers: 除外するロガー名のリスト

    Returns:
        GCloudLoggingHandler: 設定されたハンドラー
    """
    # ルートロガーを取得
    root_logger = std_logging.getLogger()

    # GCloudLoggingHandlerを作成
    handler = GCloudLoggingHandler(
        project_id=project_id,
        log_name=log_name,
        labels=labels,
        excluded_loggers=excluded_loggers,
    )

    # ログレベルを設定
    handler.setLevel(level)

    # ルートロガーにハンドラーを追加
    root_logger.addHandler(handler)

    return handler


# Simple usage example
def create_handler(
    project_id: Optional[str] = None,
    log_name: str = "python",
    labels: Optional[Dict[str, str]] = None,
) -> GCloudLoggingHandler:
    """Create a GCloudLoggingHandler with minimal configuration.

    Args:
        project_id (str, optional): Google Cloud Project ID. If not provided, it will be
            determined from the environment.
        log_name (str, optional): Name of the log to write to. Defaults to 'python'.
        labels (Dict[str, str], optional): Labels to add to all log entries.

    Returns:
        GCloudLoggingHandler: The created handler.
    """
    # Use environment variable if project_id is not provided
    if project_id is None:
        project_id = os.environ.get("GCP_PROJECT_ID")

    # Create the handler
    return GCloudLoggingHandler(
        project_id=project_id,
        log_name=log_name,
        labels=labels,
    )



"""
Handlers module for logkiss logging.

Copyright (c) 2025 Taka Suzuki
SPDX-License-Identifier: MIT
See LICENSE for details.

This module contains various handlers used in logkiss.
Main classes:
- BaseHandler: Base handler class for implementing custom handlers
- AWSCloudWatchHandler: Handler for sending logs to AWS CloudWatch Logs
"""

import json
import threading
import time
import logging
from concurrent.futures import ThreadPoolExecutor
from typing import Any, Dict, List, Optional, Union


class BaseHandler(logging.Handler):
    """Base handler class for implementing custom handlers"""

    def __init__(self, level=logging.NOTSET):
        """Initialize the handler"""
        super().__init__(level)

    def handle(self, record):
        """Handle a log record"""
        raise NotImplementedError("handle method must be implemented")


class AWSCloudWatchHandler(logging.Handler):
    """
    AWS CloudWatch Logs handler

    CloudWatchにログを送信するためのハンドラー。boto3モジュールが必要です。
    実際のインポートと初期化は実際に使用されるまで遅延されます。
    """

    def __init__(
        self,
        log_group_name: str,
        log_stream_name: Optional[str] = None,
        region_name: Optional[str] = None,
        batch_size: int = 100,
        flush_interval: float = 5.0,
    ) -> None:
        """初期化処理は実際の実装クラスに委譲します"""
        # 先に基底クラスの初期化
        super().__init__()

        # boto3が利用可能か確認
        try:
            import boto3
        except ImportError:
            raise ImportError("boto3 package is required. " "Install it with: pip install 'logkiss[cloud]'")

        # 属性を初期化して、初期化失敗時のエラーを防ぐ
        self._batch = []
        self._batch_lock = threading.Lock()
        self._batch_size = batch_size
        self._flush_interval = flush_interval
        self._sequence_token = None
        self._executor = None
        self._running = False
        self._flush_thread = None

        try:
            # AWS CloudWatch Logsクライアントを初期化
            self.client = boto3.client("logs", region_name=region_name)
            self.log_group_name = log_group_name

            if log_stream_name is None:
                # Try to determine an appropriate log stream name
                # First check if we're running on EC2
                log_stream_name = self._get_instance_identifier()

            self.log_stream_name = log_stream_name
            self._ensure_log_group_and_stream()

            # 定期的なフラッシュを開始
            self._start_periodic_flush()
        except Exception as e:
            import sys

            print(f"Error initializing AWSCloudWatchHandler: {e}", file=sys.stderr)
            # 初期化に失敗した場合は、runningフラグをFalseにして、スレッドが開始されないようにする
            self._running = False
            raise

    def _get_instance_identifier(self) -> str:
        """
        Get instance identifier for log stream name
        """
        try:
            # Try to get EC2 instance ID
            import requests

            response = requests.get("http://169.254.169.254/latest/meta-data/instance-id", timeout=0.1)
            if response.status_code == 200:
                return response.text
        except Exception:
            pass

        # Fallback to hostname
        import socket

        return socket.gethostname()

    def _ensure_log_group_and_stream(self) -> None:
        """
        Ensure log group and stream exist
        """
        # Create log group if it doesn't exist
        try:
            self.client.create_log_group(logGroupName=self.log_group_name)
        except self.client.exceptions.ResourceAlreadyExistsException:
            pass

        # Create log stream if it doesn't exist
        try:
            self.client.create_log_stream(logGroupName=self.log_group_name, logStreamName=self.log_stream_name)
        except self.client.exceptions.ResourceAlreadyExistsException:
            # Log stream already exists, get the sequence token
            response = self.client.describe_log_streams(logGroupName=self.log_group_name, logStreamNamePrefix=self.log_stream_name, limit=1)

            if response.get("logStreams"):
                self._sequence_token = response["logStreams"][0].get("uploadSequenceToken")

    def _start_periodic_flush(self) -> None:
        """Start a background thread to periodically flush the batch."""
        self._running = True
        self._flush_thread = threading.Thread(target=self._periodic_flush_worker, daemon=True)
        self._flush_thread.start()

    def _periodic_flush_worker(self) -> None:
        """Worker function for the periodic flush thread."""
        while self._running:
            try:
                # Sleep for the specified interval
                time.sleep(self._flush_interval)

                # バッチが空でなければフラッシュ
                if self._batch:
                    self._flush()
            except Exception as e:
                import sys

                print(f"Error in periodic flush: {e}", file=sys.stderr)

    def emit(self, record: logging.LogRecord) -> None:
        """Process log record"""
        try:
            # フォーマット済みのメッセージを取得
            msg = self.format(record)

            # タイムスタンプ（ミリ秒単位）
            timestamp = int(record.created * 1000)

            # バッチに追加するエントリ
            entry = {"timestamp": timestamp, "message": msg}

            # exc_info=Trueが指定された場合のスタックトレース情報を追加
            if record.exc_info:
                import traceback
                import json

                # JSONとして追加情報を埋め込む
                entry["message"] += "\nStack Trace: " + json.dumps({"stack_trace": traceback.format_exception(*record.exc_info)})

            # バッチに追加
            with self._batch_lock:
                self._batch.append(entry)

                # バッチサイズに達したらフラッシュ
                if len(self._batch) >= self._batch_size:
                    self._flush()
        except Exception as e:
            import sys

            print(f"Error in AWSCloudWatchHandler.emit: {e}", file=sys.stderr)

    def _flush(self) -> None:
        """Flush batch"""
        if not self._running:
            return

        entries = []
        with self._batch_lock:
            if not self._batch:
                return

            entries = self._batch
            self._batch = []

        if not entries:
            return

        # Sort entries by timestamp
        entries.sort(key=lambda x: x["timestamp"])

        # Convert to CloudWatch Logs format
        log_events = [{"timestamp": entry["timestamp"], "message": entry["message"]} for entry in entries]

        # ここでLazy Importを行う - 実際にAWS操作が必要なときだけ
        try:
            import boto3

            # Send to CloudWatch Logs
            kwargs = {"logGroupName": self.log_group_name, "logStreamName": self.log_stream_name, "logEvents": log_events}

            if self._sequence_token:
                kwargs["sequenceToken"] = self._sequence_token

            response = self.client.put_log_events(**kwargs)
            self._sequence_token = response.get("nextSequenceToken")
        except Exception as e:
            if hasattr(e, "__class__") and e.__class__.__name__ == "InvalidSequenceTokenException":
                # Get the correct sequence token from the error message
                import re

                match = re.search(r"sequenceToken is: (\S+)", str(e))
                if match:
                    self._sequence_token = match.group(1)
                    # Retry with the correct sequence token
                    self._flush()
            else:
                import sys

                print(f"Error writing to CloudWatch Logs: {e}", file=sys.stderr)
                # Put the entries back in the batch
                with self._batch_lock:
                    self._batch = entries + self._batch

    def flush(self) -> None:
        """Flush all queued messages to CloudWatch Logs"""
        self._flush()

    def close(self):
        """
        Close the handler and release all resources.
        """
        if not hasattr(self, "_running"):
            # 初期化が完了していない場合は何もしない
            super().close()
            return

        try:
            # スレッドを停止
            self._running = False

            # 最後の一回フラッシュを試みる
            try:
                self._flush()
            except Exception as e:
                import sys

                print(f"Error in final flush: {e}", file=sys.stderr)

            # スレッドが存在し、実行中であれば、終了を待つ（最大1秒）
            if hasattr(self, "_flush_thread") and self._flush_thread is not None:
                if self._flush_thread.is_alive():
                    self._flush_thread.join(timeout=1.0)
        except Exception as e:
            import sys

            print(f"Error closing handler: {e}", file=sys.stderr)
        finally:
            # 親クラスのcloseメソッドを呼び出す
            super().close()

    def __del__(self):
        """Cleanup when the handler is deleted"""
        try:
            if hasattr(self, "_running") and self._running:
                self.close()
        except Exception:
            # __del__内では例外を無視
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
logkissの設定モジュール

標準のloggingモジュールの設定関数を拡張し、色付きログなどの機能をサポートします。
"""

import logging
import logging.config
import os
import sys
from typing import Dict, Any, Optional, Union
from pathlib import Path
import yaml
from yaml import safe_load
from yaml.error import YAMLError

# 循環インポートを避けるために関数内でインポートする
def _get_colored_formatter():
    from logkiss.logkiss import ColoredFormatter
    return ColoredFormatter


def dictConfig(config: Dict[str, Any]) -> None:
    """
    Configure logging using a dictionary

    This function is similar to logging.config.dictConfig, but it also handles
    color settings for ColoredFormatter.

    Args:
        config: A dictionary containing logging configuration

    Example:
        >>> import logkiss.config
        >>> logkiss.config.dictConfig({
        ...     "version": 1,
        ...     "formatters": {
        ...         "colored": {
        ...             "format": "%(asctime)s [%(levelname)s] %(message)s",
        ...             "datefmt": "%Y-%m-%d %H:%M:%S",
        ...             "class": "logkiss.ColoredFormatter",
        ...             "colors": {
        ...                 "levels": {
        ...                     "DEBUG": {"fg": "blue"},
        ...                     "INFO": {"fg": "white"},
        ...                     "WARNING": {"fg": "yellow"},
        ...                     "ERROR": {"fg": "black", "bg": "red"},
        ...                     "CRITICAL": {"fg": "black", "bg": "bright_red", "style": "bold"}
        ...                 }
        ...             }
        ...         }
        ...     },
        ...     "handlers": {
        ...         "console": {
        ...             "class": "logging.StreamHandler",
        ...             "level": "DEBUG",
        ...             "formatter": "colored",
        ...             "stream": "ext://sys.stdout"
        ...         }
        ...     },
        ...     "root": {
        ...         "level": "DEBUG",
        ...         "handlers": ["console"]
        ...     }
        ... })
    """
    # Get color settings in advance
    color_configs = {}
    if "formatters" in config:
        for formatter_name, formatter_config in config["formatters"].items():
            if "colors" in formatter_config:
                # ディープコピーを作成して完全に置き換え
                import copy
                color_configs[formatter_name] = copy.deepcopy(formatter_config["colors"])
    
    # logging.config.dictConfigを使用して設定を適用
    logging.config.dictConfig(config)
    
    # 色設定を適用
    if color_configs:
        # 対応するフォーマッタを取得
        ColoredFormatter = _get_colored_formatter()
        
        # ルートロガーのハンドラーを取得
        root_logger = logging.getLogger("")
        if root_logger.handlers:
            # 全てのハンドラーに対して処理
            for handler in root_logger.handlers:
                if hasattr(handler, 'formatter') and isinstance(handler.formatter, ColoredFormatter):
                    formatter = handler.formatter
                    
                    # フォーマッター名を特定
                    for formatter_name, color_config in color_configs.items():
                        # 色設定を完全に置き換え
                        # ColorManagerの設定を更新
                        if hasattr(formatter, 'color_manager'):
                            formatter.color_manager.config = color_config
                        
                        # 一度適用したら終了
                        break


def fileConfig(fname: Union[str, Path], defaults: Optional[Dict[str, str]] = None, 
               disable_existing_loggers: bool = True) -> None:
    """
    標準ロギングライブラリのlogging.config.fileConfigと互換性のある設定関数

    この関数は標準のlogging.config.fileConfigを呼び出し、その後にlogkissの
    色設定を適用します。

    Args:
        fname: 設定ファイルのパス
        defaults: デフォルト値の辞書
        disable_existing_loggers: 既存のロガーを無効化するかどうか

    Note:
        現在のバージョンでは、標準のfileConfig形式での色設定はサポートされていません。
        色設定を行う場合は、YAMLファイルとdictConfigを使用してください。
    """
    # 標準のfileConfigを呼び出し
    logging.config.fileConfig(fname, defaults, disable_existing_loggers)


def load_yaml_config(config_path: Union[str, Path]) -> Dict[str, Any]:
    """
    YAMLファイルから設定を読み込む

    Args:
        config_path: 設定ファイルのパス

    Returns:
        設定辞書

    Raises:
        ValueError: 設定ファイルが存在しない場合
        yaml.YAMLError: YAMLの形式が不正な場合
    """
    try:
        with open(config_path, "r", encoding='utf-8') as f:
            config = safe_load(f)
    except FileNotFoundError as exc:
        raise ValueError(f"Configuration file not found: {config_path}") from exc
    except yaml.YAMLError as exc:
        raise yaml.YAMLError(f"Invalid YAML format in {config_path}: {exc}")
    
    # 空の設定ファイルの場合はデフォルト設定を返す
    if config is None:
        return {"version": 1}
    
    return config


def yaml_config(config_path: Union[str, Path]) -> None:
    """
    YAMLファイルから設定を読み込み、dictConfigで適用する

    環境変数が設定されている場合は、それらがYAMLファイルの設定よりも優先されます。

    Args:
        config_path: 設定ファイルのパス

    Example:
        >>> import logkiss.config
        >>> logkiss.config.yaml_config("logging_config.yaml")
    """
    # YAMLファイルから設定を読み込む
    config = load_yaml_config(config_path)
    
    # 最小限の設定が含まれていることを確認
    if "version" not in config:
        config["version"] = 1
    
    # 基本的なロギング設定がない場合は追加
    if "handlers" not in config:
        config["handlers"] = {
            "console": {
                "class": "logkiss.KissConsoleHandler",
                "level": "WARNING",
                "formatter": "colored"
            }
        }
    
    if "formatters" not in config:
        config["formatters"] = {
            "colored": {
                "class": "logkiss.ColoredFormatter",
                "format": "%(asctime)s [%(levelname)s] %(message)s"
            }
        }
    
    if "root" not in config:
        config["root"] = {
            "level": "WARNING",
            "handlers": ["console"]
        }
    
    # 環境変数から設定を取得
    level_env = os.environ.get("LOGKISS_LEVEL")
    fmt_env = os.environ.get("LOGKISS_FORMAT")
    datefmt_env = os.environ.get("LOGKISS_DATEFMT")
    
    # ログレベルを文字列から数値に変換
    if level_env:
        level_map = {
            "DEBUG": logging.DEBUG,
            "INFO": logging.INFO,
            "WARNING": logging.WARNING,
            "ERROR": logging.ERROR,
            "CRITICAL": logging.CRITICAL
        }
        level_value = level_map.get(level_env.upper(), logging.WARNING)
    
    # 環境変数が設定されている場合は、それらを設定に反映
    if level_env:
        # ルートロガーのレベルを設定
        if "loggers" in config and "" in config["loggers"]:
            config["loggers"][""]["level"] = level_value
        # rootセクションがある場合はそちらも設定
        if "root" in config:
            config["root"]["level"] = level_value
        # ハンドラーのレベルも設定
        if "handlers" in config:
            for handler in config["handlers"].values():
                handler["level"] = level_value
    
    # フォーマット文字列を設定
    if fmt_env and "formatters" in config:
        for formatter in config["formatters"].values():
            formatter["format"] = fmt_env
    
    # 日付フォーマット文字列を設定
    if datefmt_env and "formatters" in config:
        for formatter in config["formatters"].values():
            formatter["datefmt"] = datefmt_env
    
    # 設定を適用
    dictConfig(config)


def find_config_file() -> Optional[Path]:
    """
    設定ファイルを探す

    環境変数LOGKISS_CONFIGで指定されたパス、またはデフォルトの場所から設定ファイルを探します。

    Returns:
        設定ファイルのパス、見つからない場合はNone
    """
    # 環境変数から設定ファイルのパスを取得
    env_path = os.environ.get("LOGKISS_CONFIG")
    if env_path:
        p = Path(env_path).expanduser()
        if p.exists():
            return p

    # デフォルトの場所を探す
    candidates = []
    if sys.platform.startswith("win"):
        appdata = os.environ.get("APPDATA")
        userprofile = os.environ.get("USERPROFILE")
        if appdata:
            candidates.append(Path(appdata) / "logkiss" / "config.yaml")
        if userprofile:
            candidates.append(Path(userprofile) / ".config" / "logkiss" / "config.yaml")
    else:
        home = Path.home()
        candidates.append(home / ".config" / "logkiss" / "config.yaml")
        candidates.append(Path.cwd() / "logkiss.yaml")

    for candidate in candidates:
        if candidate.exists():
            return candidate

    return None


def auto_config() -> None:
    """
    環境変数と設定ファイルから自動的に設定を適用する

    環境変数LOGKISS_SKIP_CONFIGが指定されている場合は設定ファイルの読み込みをスキップします。
    それ以外の場合は、設定ファイルを探して適用します。
    設定ファイルが見つからない場合は、環境変数から設定を適用します。
    """
    # 設定ファイルの読み込みをスキップするかチェック
    skip_config = os.environ.get("LOGKISS_SKIP_CONFIG", "").lower() in ("1", "true", "yes")
    if skip_config:
        # 環境変数から設定を適用
        _config_from_env()
        return

    # 設定ファイルを探す
    config_path = find_config_file()
    if config_path:
        # 設定ファイルから設定を適用
        yaml_config(config_path)
    else:
        # 環境変数から設定を適用
        _config_from_env()


def _config_from_env() -> None:
    """
    環境変数から設定を適用する

    環境変数:
        LOGKISS_LEVEL: ログレベル (DEBUG, INFO, WARNING, ERROR, CRITICAL)
        LOGKISS_FORMAT: ログフォーマット文字列
        LOGKISS_DATEFMT: 日付フォーマット文字列
        LOGKISS_DISABLE_COLOR: カラー出力を無効化する場合は'true'
        NO_COLOR: 業界標準の環境変数（値は何でもよい）
    """
    # 環境変数から設定を取得
    level = os.environ.get("LOGKISS_LEVEL", "WARNING")
    fmt = os.environ.get("LOGKISS_FORMAT", "%(asctime)s,%(msecs)03d %(levelname)-5s | %(filename)s:%(lineno)3d | %(message)s")
    datefmt = os.environ.get("LOGKISS_DATEFMT", "%Y-%m-%d %H:%M:%S")

    # カラー出力の設定
    # カラー出力の設定はフォーマッター内で行われるので、ここでは特に設定しない
    
    # 環境変数の確認はフォーマッター内で行われる
    # 1. LOGKISS_DISABLE_COLOR: このライブラリ固有の環境変数
    # 2. NO_COLOR: 業界標準の環境変数 (https://no-color.org/)

    # dictConfig用の設定辞書を作成
    config = {
        "version": 1,
        "formatters": {
            "colored": {
                "()": "logkiss.ColoredFormatter",
                "format": fmt,
                "datefmt": datefmt,
                "colors": {
                    "levels": {
                        "DEBUG": {"fg": "blue"},
                        "INFO": {"fg": "white"},
                        "WARNING": {"fg": "yellow"},
                        "ERROR": {"fg": "black", "bg": "red"},
                        "CRITICAL": {"fg": "black", "bg": "bright_red", "style": "bold"},
                    }
                }
            }
        },
        "handlers": {
            "console": {
                "class": "logkiss.KissConsoleHandler",
                "level": level,
                "formatter": "colored"
            }
        },
        "loggers": {
            "": {
                "handlers": ["console"],
                "level": level
            }
        }
    }

    # 設定を適用
    dictConfig(config)


def _auto_config_from_env() -> None:
    """
    環境変数から自動的に設定を適用する

    KissLogger.reload_config()メソッドから呼び出されるための内部関数です。
    """
    _config_from_env()


# モジュール読み込み時に自動的に設定を適用
# ただし、引数なしでimportした場合のみ
# 例: import logkiss
# 例外: from logkiss import getLogger
if __name__ != "__main__" and not hasattr(logging.getLogger(), "_logkiss_configured"):
    auto_config()
    # 設定済みフラグを設定
    setattr(logging.getLogger(), "_logkiss_configured", True)


# 後方互換性のために残す（非推奨）
config_from_env = _config_from_env
